var page = this;
var requestCounterMinWaitMs = $$REQUEST_COUNTER_MIN_WAIT$$;
var requestCounterMaxWaitMs = $$REQUEST_COUNTER_MAX_WAIT$$;
var readyStateInterval = null;
var checkAjaxCompleteInterval = null;
var startTime = new Date().getTime();
var lastReceived = new Date().getTime();
var requestIds = [];
var requestCount = 0;
var responseCount = 0;
var url = null;
var NONE = 0;
var ERROR = 1;
var WARN = 2;
var INFO = 3;
var DEBUG = 4;
var logLevelStr = '$$LOG_LEVEL$$';
var logLevel = INFO;
if (!logLevelStr) {
  logLevelStr = 'info';
} else {
  switch (logLevelStr.toLowerCase()) {
    case "none":
        logLevel = NONE;
        break;
    case "error":
        logLevel = ERROR;
        break;
    case "warn":
        logLevel = WARN;
        break;
    case "info":
        logLevel = INFO;
        break;
    case "debug":
        logLevel = DEBUG;
        break;
    default:
        logLevel = INFO;
        break;
  }
}
var isTimedOut = function() {
  return new Date().getTime() - startTime > requestCounterMaxWaitMs;
}
var log = function(level, msg) {
  if (logLevel >= level) {
    console.log('[' + logLevelStr.toUpperCase() + '] - ' + msg + '\n');
  }
}
var checkAjaxComplete = function() {
  var isDone = (new Date().getTime() - lastReceived > requestCounterMinWaitMs && requestCount === responseCount);
  if (isDone || isTimedOut())  {
    if (isDone) {
      log(INFO, 'Finished loading ' + url + ' requestCount=' + requestCount + ', and responseCount=' + responseCount);
    } else {
      log(INFO, 'Timed out waiting for ajax from ' + url + ' with requestCount=' + requestCount + ', and responseCount=' + responseCount);
    }
    var setLwPageLoadVarScript = "function(){ window.lwFusionPageLoadVar='true'; }";
    page.evaluateJavaScript(setLwPageLoadVarScript);
    clearInterval(checkAjaxCompleteInterval);
  } else {
    log(DEBUG, 'Still waiting for ajax to load for ' + url + ' requestCount=' + requestCount + ', and responseCount=' + responseCount + '. Time since start = ' + (new Date().getTime() - startTime));
  }
}
var checkReadyState = function() {
  var readyState = page.evaluate(function () {
    return document.readyState;
  });
  if ("complete" === readyState) {
    log(DEBUG, 'document.readyState = ready. Time since start = ' + (new Date().getTime() - startTime) + '. Checking ajax complete now');
    clearInterval(readyStateInterval);
    checkAjaxCompleteInterval = setInterval(checkAjaxComplete, 500);
  } else if (isTimedOut()) {
    clearInterval(readyStateInterval);
    log(INFO, 'Timed out waiting for completion status from ' + url + ' with requestCount=' + requestCount + ', and responseCount=' + responseCount);
  } else {
    log(DEBUG, 'Still waiting for ready state. Time since start = ' + (new Date().getTime() - startTime));
  }
}
page.onResourceReceived = function (response) {
  if (requestIds.indexOf(response.id) !== -1) {
    lastReceived = new Date().getTime();
    responseCount++;
    requestIds[requestIds.indexOf(response.id)] = null;
    log(DEBUG, 'onResourceReceived - current counts: requestCount=' + requestCount + ', and responseCount=' + responseCount)
  }
};

page.onResourceRequested = function (request) {
  if(requestIds.indexOf(request.id) === -1) {
    requestIds.push(request.id);
    requestCount++;
    log(DEBUG, 'onResourceRequested - current counts: requestCount=' + requestCount + ', and responseCount=' + responseCount);
  }
};
page.onInitialized = function () {
  readyStateInterval = null;
  checkAjaxCompleteInterval = null;
  startTime = new Date().getTime();
  lastReceived = new Date().getTime();
  requestIds = [];
  requestCount = 0;
  responseCount = 0;
  url = page.url;
  readyStateInterval = setInterval(checkReadyState, 250);
}